/*
 * Copyright (c) 2015, 2016, 2017 Adrian Siekierka
 *
 * This file is part of Preston.
 *
 * Preston is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Preston is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Preston.  If not, see <http://www.gnu.org/licenses/>.
 */

package pl.asie.preston.container;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.util.ITooltipFlag;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.NonNullList;
import net.minecraft.util.text.translation.I18n;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pl.asie.preston.PrestonMod;
import pl.asie.preston.util.PrestonUtils;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.List;

public class ItemCompressedBlock extends ItemBlock {

	public ItemCompressedBlock(Block b) {
		super(b);
		func_77637_a(PrestonMod.CREATIVE_TAB);
	}

	public static boolean canCompress(ItemStack stack) {
		if (stack.func_77973_b() instanceof ItemCompressedBlock) {
			return getLevel(stack) < PrestonMod.MAX_COMPRESSION_LEVELS;
		} else {
			if (!PrestonMod.whitelistedItems.isEmpty()) {
				if (!PrestonMod.whitelistedItems.contains(stack.func_77973_b())) {
					return false;
				}
			}

			if (PrestonMod.blacklistedItems.contains(stack.func_77973_b())) {
				return false;
			}

			for (ItemStack other : PrestonMod.blacklistedItemStacks) {
				if (PrestonUtils.canMerge(stack, other)) {
					return false;
				}
			}

			IBlockState state = PrestonUtils.getBlockState(stack);
			if (state != null) {
				if (state.func_185904_a() == Material.field_151579_a) {
					return false;
				}

				if (state.func_185917_h() || state.func_185913_b()) {
					return true;
				}

				try {
					if (state.func_185900_c(new FakeBlockAccess(NullBlockAccess.INSTANCE, FakeBlockAccess.ACCESS_POS, state), FakeBlockAccess.ACCESS_POS).equals(Block.field_185505_j)) {
						return true;
					}
				} catch (Exception e) {

				}

				return false;
			}
		}

		return false;
	}

	public static boolean canDecompress(ItemStack stack) {
		return stack.func_77973_b() instanceof ItemCompressedBlock;
	}

	public static ItemStack getContained(ItemStack stack) {
		if (stack.func_77973_b() instanceof ItemCompressedBlock) {
			return new ItemStack(PrestonUtils.getTagCompound(stack, true).func_74775_l("stack"));
		} else {
			return ItemStack.field_190927_a;
		}
	}

	public static int getLevel(ItemStack stack) {
		if (stack.func_77973_b() instanceof ItemCompressedBlock) {
			return PrestonUtils.getTagCompound(stack, true).func_74762_e("level");
		} else {
			return 0;
		}
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void func_77624_a(ItemStack stack, @Nullable World world, List<String> tooltip, ITooltipFlag flag) {
		super.func_77624_a(stack, world, tooltip, flag);
		if (PrestonMod.ENABLE_COUNT_IN_TOOLTIPS) {
			int level = getLevel(stack);
			if (level > 0) {
				BigInteger amount = BigInteger.valueOf(PrestonMod.COMPRESSED_BLOCK_AMOUNT);
				amount = amount.pow(level);
				tooltip.add(amount + " x " + getContained(stack).func_82833_r());
			}
		}
	}

	@Override
	public void func_150895_a(CreativeTabs tab, NonNullList<ItemStack> items) {
		if (this.func_194125_a(tab)) {
			PrestonMod.proxy.getSubItems(tab, items);
		}
	}

	@Nullable
	public static ItemStack shiftLevel(ItemStack source, int amount) {
		return setLevel(source, getLevel(source) + amount);
	}

	@Nullable
	public static ItemStack setLevel(ItemStack source, int targetLevel) {
		int sourceLevel = getLevel(source);
		if (targetLevel < 0 || targetLevel > PrestonMod.MAX_COMPRESSION_LEVELS) {
			return null;
		} else if (targetLevel == sourceLevel) {
			return source.func_77946_l();
		} else if (targetLevel == 0) {
			return getContained(source);
		} else if (sourceLevel == 0) {
			ItemStack target = new ItemStack(PrestonMod.itemCompressedBlock, 1, 0);
			NBTTagCompound compound = PrestonUtils.getTagCompound(target, true);
			ItemStack sourceS = source;
			if (sourceS.func_190916_E() > 1) {
				sourceS = sourceS.func_77946_l();
				sourceS.func_190920_e(1);
			}

			compound.func_74782_a("stack", sourceS.serializeNBT());
			compound.func_74768_a("level", targetLevel);
			return target;
		} else {
			ItemStack target = source.func_77946_l();
			target.func_190920_e(1);
			NBTTagCompound compound = PrestonUtils.getTagCompound(target, true);
			compound.func_74768_a("level", targetLevel);
			return target;
		}
	}

	@Override
	public String func_77653_i(ItemStack stack) {
		String level = "preston.tuple." + getLevel(stack);
		String name = getContained(stack).func_82833_r();
		return I18n.func_74837_a("item.preston.compressed_block.name", I18n.func_74838_a(level), name);
	}
}
